@@ -1,5 +1,6 @@
 #include <ctype.h>
 #include <stdarg.h>
+#include <stdint.h> // for intptr_t
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> // for memcmp()
@@ -499,7 +500,7 @@ static int generic_is0(element_ptr a) {
   int result;
   element_t b;
   element_init(b, a->field);
-  result = element_cmp(a, b);
+  result = !element_cmp(a, b); // element_cmp returns 0 if 'a' and 'b' are the same, nonzero otherwise. generic_is0 returns true if 'a' is 0.
   element_clear(b);
   return result;
 }
@@ -509,7 +510,7 @@ static int generic_is1(element_ptr a) {
   element_t b;
   element_init(b, a->field);
   element_set1(b);
-  result = element_cmp(a, b);
+  result = !element_cmp(a, b); // element_cmp returns 0 if 'a' and 'b' are the same, nonzero otherwise. generic_is1 returns true if 'a' is 1.
   element_clear(b);
   return result;
 }
@@ -754,8 +755,12 @@ int pbc_mpz_set_str(mpz_t z, const char *s, int base) {
 // Divides `n` with primes up to `limit`. For each factor found,
 // call `fun`. If the callback returns nonzero, then aborts and returns 1.
 // Otherwise returns 0.
-int pbc_trial_divide(int (*fun)(mpz_t factor, unsigned int multiplicity),
-    mpz_t n, mpz_ptr limit) {
+int pbc_trial_divide(int (*fun)(mpz_t factor,
+                                unsigned int multiplicity,
+                                void *scope_ptr),
+                     void *scope_ptr,
+                     mpz_t n,
+                     mpz_ptr limit) {
   mpz_t p, m;
   mpz_t fac;
   unsigned int mul;
@@ -780,7 +785,7 @@ int pbc_trial_divide(int (*fun)(mpz_t factor, unsigned int multiplicity),
         mpz_divexact(m, m, p);
         mul++;
       } while (mpz_divisible_p(m, p));
-      if (fun(fac, mul)) {
+      if (fun(fac, mul, scope_ptr)) {
         mpz_clear(fac);
         mpz_clear(m);
         mpz_clear(p);
