@@ -2,6 +2,10 @@
 
 # this is all from my ebuild (no doi)
 
+function die() {
+    echo mv_things.sh ERROR
+    exit 1
+}
 
 if [ -z "$1" ]; then
     echo "no dest dir given?"
@@ -25,12 +29,12 @@ rm ${Ex}/src/*.readline.c || die
 find ${Ex} -type d -exec chmod 755 {} \; || die
 find ${Ex} -type f -exec chmod 644 {} \; || die
 
-install -o 0 -g 0 -m 644 exmakefile ${Ex}/src/Makefile
+#install -o 0 -g 0 -m 644 exmakefile ${Ex}/src/Makefile
 install -o 0 -g 0 -m 755 -d ${D}/usr/bin/ || die
 
 echo "building a real pbc"
 (cd pbc; gcc -c pbc_getline.readline.c)
-gcc -o realpbc -I. -Iinclude pbc/pbc.c -L .libs -lpbc pbc/pbc_getline.readline.o  -lreadline
+gcc -o realpbc -I. -Iinclude pbc/pbc.c -L .libs -lpbc pbc/pbc_getline.readline.o  -lreadline  pbc_pbc-symtab.o pbc_pbc-parser.tab.o pbc_pbc-darray.o pbc_pbc-lex.yy.o
 
 echo "installing the pbc binary"
 install -o 0 -g 0 -m 755 realpbc ${D}/usr/bin/pbc || die
