@@ -1,6 +1,7 @@
 #include <ctype.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdint.h> // for intptr_t
 #include <stdlib.h>
 #include <string.h>
 #include <gmp.h>
@@ -208,7 +209,7 @@ static void curve_mul(element_ptr c, element_ptr a, element_ptr b) {
 //compute c_i=a_i+a_i at one time.
 static void multi_double(element_ptr c[], element_ptr a[], int n) {
   int i;
-  element_t* table=malloc(sizeof(element_t)*n);  //a big problem?
+  element_t* table = pbc_malloc(sizeof(element_t)*n);  //a big problem?
   element_t e0, e1, e2;
   point_ptr q, r;
   curve_data_ptr cdp = a[0]->field->data;
@@ -276,13 +277,13 @@ static void multi_double(element_ptr c[], element_ptr a[], int n) {
   for(i=0; i<n; i++){
     element_clear(table[i]);
   }
-  free(table);
+  pbc_free(table);
 }
 
 //compute c_i=a_i+b_i at one time.
 static void multi_add(element_ptr c[], element_ptr a[], element_ptr b[], int n){
   int i;
-  element_t* table=malloc(sizeof(element_t)*n);  //a big problem?
+  element_t* table = pbc_malloc(sizeof(element_t)*n);  //a big problem?
   point_ptr p, q, r;
   element_t e0, e1, e2;
   curve_data_ptr cdp = a[0]->field->data;
@@ -360,7 +361,7 @@ static void multi_add(element_ptr c[], element_ptr a[], element_ptr b[], int n){
   for(i=0; i<n; i++){
     element_clear(table[i]);
   }
-  free(table);
+  pbc_free(table);
 }
 
 
@@ -502,9 +503,9 @@ static int curve_snprint(char *s, size_t n, element_ptr a) {
   size_t result = 0, left;
   int status;
 
-  void clip_sub(void) {
-    result += status;
-    left = result >= n ? 0 : n - result;
+  #define clip_sub() {                   \
+    result += status;                    \
+    left = result >= n ? 0 : n - result; \
   }
 
   if (p->inf_flag) {
@@ -528,6 +529,7 @@ static int curve_snprint(char *s, size_t n, element_ptr a) {
   status = snprintf(s + result, left, "]");
   if (status < 0) return status;
   return result + status;
+  #undef clip_sub
 }
 
 static void curve_set_multiz(element_ptr a, multiz m) {
