@@ -2,6 +2,7 @@
 
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdint.h> // for intptr_t
 #include <stdlib.h>
 #include <string.h>
 #include <gmp.h>
@@ -127,141 +128,141 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, element_t P,
   const element_ptr Px = curve_x_coord(P);
   const element_ptr Py = curve_y_coord(P);
 
-  void proj_double(void) {
-    // t0 = 3x^2 + (curve_a) z^4
-    element_square(t0, Zx);
-    //element_mul_si(t0, t0, 3);
-    element_double(t1, t0);
-    element_add(t0, t0, t1);
-    element_square(t1, z2);
-    element_mul(t1, t1, curve_a);
-    element_add(t0, t0, t1);
-
-    // z_out = 2 y z
-    element_mul(z, Zy, z);
-    //element_mul_si(z, z, 2);
-    element_double(z, z);
-    element_square(z2, z);
-
-    // t1 = 4 x y^2
-    element_square(t2, Zy);
-    element_mul(t1, Zx, t2);
-    //element_mul_si(t1, t1, 4);
-    element_double(t1, t1);
-    element_double(t1, t1);
-
-    // x_out = t0^2 - 2 t1
-    //element_mul_si(t3, t1, 2);
-    element_double(t3, t1);
-    element_square(Zx, t0);
-    element_sub(Zx, Zx, t3);
-
-    // t2 = 8y^4
-    element_square(t2, t2);
-    //element_mul_si(t2, t2, 8);
-    element_double(t2, t2);
-    element_double(t2, t2);
-    element_double(t2, t2);
-
-    // y_out = t0(t1 - x_out) - t2
-    element_sub(t1, t1, Zx);
-    element_mul(t0, t0, t1);
-    element_sub(Zy, t0, t2);
+  #define proj_double() {             \
+    /* t0 = 3x^2 + (curve_a) z^4 */   \
+    element_square(t0, Zx);           \
+    /* element_mul_si(t0, t0, 3); */  \
+    element_double(t1, t0);           \
+    element_add(t0, t0, t1);          \
+    element_square(t1, z2);           \
+    element_mul(t1, t1, curve_a);     \
+    element_add(t0, t0, t1);          \
+                                      \
+    /* z_out = 2 y z */               \
+    element_mul(z, Zy, z);            \
+    /* element_mul_si(z, z, 2); */    \
+    element_double(z, z);             \
+    element_square(z2, z);            \
+                                      \
+    /* t1 = 4 x y^2 */                \
+    element_square(t2, Zy);           \
+    element_mul(t1, Zx, t2);          \
+    /* element_mul_si(t1, t1, 4); */  \
+    element_double(t1, t1);           \
+    element_double(t1, t1);           \
+                                      \
+    /* x_out = t0^2 - 2 t1 */         \
+    /* element_mul_si(t3, t1, 2); */  \
+    element_double(t3, t1);           \
+    element_square(Zx, t0);           \
+    element_sub(Zx, Zx, t3);          \
+                                      \
+    /* t2 = 8y^4 */                   \
+    element_square(t2, t2);           \
+    /* element_mul_si(t2, t2, 8); */  \
+    element_double(t2, t2);           \
+    element_double(t2, t2);           \
+    element_double(t2, t2);           \
+                                      \
+    /* y_out = t0(t1 - x_out) - t2 */ \
+    element_sub(t1, t1, Zx);          \
+    element_mul(t0, t0, t1);          \
+    element_sub(Zy, t0, t2);          \
   }
 
-  void proj_mixin(void) {
-    // t2 = Px z^2
-    element_mul(t2, z2, Px);
-
-    // t3 = Zx - t2
-    element_sub(t3, Zx, t2);
-
-    // t0 = Py z^3
-    element_mul(t0, z2, Py);
-    element_mul(t0, t0, z);
-
-    // t1 = Zy - t0
-    element_sub(t1, Zy, t0);
-
-    // e7 = Zx + t2, use t2 to double for e7
-    element_add(t2, Zx, t2);
-
-    // e8 = Zy + t0, use t0 to double for e8
-    element_add(t0, Zy, t0);
-
-    // z = z t3
-    element_mul(z, z, t3);
-    element_square(z2, z);
-
-    // Zx = t1^2 - e7 t3^2
-    // t3 now holds t3^3,
-    // t4 holds e7 t3^2.
-    element_square(t4, t3);
-    element_mul(t3, t4, t3);
-    element_square(Zx, t1);
-    element_mul(t4, t2, t4);
-    element_sub(Zx, Zx, t4);
-
-    // t4 = e7 t3^2 - 2 Zx
-    element_sub(t4, t4, Zx);
-    element_sub(t4, t4, Zx);
-
-    // Zy = (t4 t1 - e8 t3^3)/2
-    element_mul(t4, t4, t1);
-    element_mul(t0, t0, t3);
-    element_sub(t4, t4, t0);
-    element_halve(Zy, t4);
+  #define proj_mixin() {                        \
+    /* t2 = Px z^2 */                           \
+    element_mul(t2, z2, Px);                    \
+                                                \
+    /* t3 = Zx - t2 */                          \
+    element_sub(t3, Zx, t2);                    \
+                                                \
+    /* t0 = Py z^3 */                           \
+    element_mul(t0, z2, Py);                    \
+    element_mul(t0, t0, z);                     \
+                                                \
+    /* t1 = Zy - t0 */                          \
+    element_sub(t1, Zy, t0);                    \
+                                                \
+    /* e7 = Zx + t2, use t2 to double for e7 */ \
+    element_add(t2, Zx, t2);                    \
+                                                \
+    /* e8 = Zy + t0, use t0 to double for e8 */ \
+    element_add(t0, Zy, t0);                    \
+                                                \
+    /* z = z t3 */                              \
+    element_mul(z, z, t3);                      \
+    element_square(z2, z);                      \
+                                                \
+    /* Zx = t1^2 - e7 t3^2 */                   \
+    /* t3 now holds t3^3, */                    \
+    /* t4 holds e7 t3^2. */                     \
+    element_square(t4, t3);                     \
+    element_mul(t3, t4, t3);                    \
+    element_square(Zx, t1);                     \
+    element_mul(t4, t2, t4);                    \
+    element_sub(Zx, Zx, t4);                    \
+                                                \
+    /* t4 = e7 t3^2 - 2 Zx */                   \
+    element_sub(t4, t4, Zx);                    \
+    element_sub(t4, t4, Zx);                    \
+                                                \
+    /* Zy = (t4 t1 - e8 t3^3)/2 */              \
+    element_mul(t4, t4, t1);                    \
+    element_mul(t0, t0, t3);                    \
+    element_sub(t4, t4, t0);                    \
+    element_halve(Zy, t4);                      \
   }
 
-  void do_tangent(void) {
-    // a = -(3x^2 + cca z^4)
-    // b = 2 y z^3
-    // c = -(2 y^2 + x a)
-    // a = z^2 a
-    element_square(a, z2);
-    element_mul(a, a, curve_a);
-    element_square(b, Zx);
-    //element_mul_si(b, b, 3);
-    element_double(t0, b);
-    element_add(b, b, t0);
-    element_add(a, a, b);
-    element_neg(a, a);
-
-    element_mul(b, z, z2);
-    element_mul(b, b, Zy);
-    element_mul_si(b, b, 2);
-
-    element_mul(c, Zx, a);
-    element_mul(a, a, z2);
-    element_square(t0, Zy);
-    element_mul_si(t0, t0, 2);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    d_miller_evalfn(e0, a, b, c, Qx, Qy);
-    element_mul(v, v, e0);
+  #define do_tangent() {                  \
+    /* a = -(3x^2 + cca z^4) */           \
+    /* b = 2 y z^3 */                     \
+    /* c = -(2 y^2 + x a) */              \
+    /* a = z^2 a */                       \
+    element_square(a, z2);                \
+    element_mul(a, a, curve_a);           \
+    element_square(b, Zx);                \
+    /* element_mul_si(b, b, 3); */        \
+    element_double(t0, b);                \
+    element_add(b, b, t0);                \
+    element_add(a, a, b);                 \
+    element_neg(a, a);                    \
+                                          \
+    element_mul(b, z, z2);                \
+    element_mul(b, b, Zy);                \
+    element_mul_si(b, b, 2);              \
+                                          \
+    element_mul(c, Zx, a);                \
+    element_mul(a, a, z2);                \
+    element_square(t0, Zy);               \
+    element_mul_si(t0, t0, 2);            \
+    element_add(c, c, t0);                \
+    element_neg(c, c);                    \
+                                          \
+    d_miller_evalfn(e0, a, b, c, Qx, Qy); \
+    element_mul(v, v, e0);                \
   }
 
-  void do_line(void) {
-    // a = -(Py z^3 - Zy)
-    // b = Px z^3 - Zx z
-    // c = Zx z Py - Zy Px;
-
-    element_mul(t0, Zx, z);
-    element_mul(t1, z2, z);
-
-    element_mul(a, Py, t1);
-    element_sub(a, Zy, a);
-
-    element_mul(b, Px, t1);
-    element_sub(b, b, t0);
-
-    element_mul(t0, t0, Py);
-    element_mul(c, Zy, Px);
-    element_sub(c, t0, c);
-
-    d_miller_evalfn(e0, a, b, c, Qx, Qy);
-    element_mul(v, v, e0);
+  #define do_line() {                     \
+    /* a = -(Py z^3 - Zy) */              \
+    /* b = Px z^3 - Zx z */               \
+    /* c = Zx z Py - Zy Px; */            \
+                                          \
+    element_mul(t0, Zx, z);               \
+    element_mul(t1, z2, z);               \
+                                          \
+    element_mul(a, Py, t1);               \
+    element_sub(a, Zy, a);                \
+                                          \
+    element_mul(b, Px, t1);               \
+    element_sub(b, b, t0);                \
+                                          \
+    element_mul(t0, t0, Py);              \
+    element_mul(c, Zy, Px);               \
+    element_sub(c, t0, c);                \
+                                          \
+    d_miller_evalfn(e0, a, b, c, Qx, Qy); \
+    element_mul(v, v, e0);                \
   }
 
   element_init(a, Px->field);
@@ -309,6 +310,10 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, element_t P,
   element_clear(e0);
   element_clear(z);
   element_clear(z2);
+  #undef proj_double
+  #undef proj_mixin
+  #undef do_tangent
+  #undef do_line
 }
 
 // Same as above, but with affine coordinates.
@@ -333,42 +338,42 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, element_t P,
   }
   */
 
-  void do_tangent(void) {
-    // a = -(3 Zx^2 + cc->a)
-    // b = 2 * Zy
-    // c = -(2 Zy^2 + a Zx);
-
-    element_square(a, Zx);
-    element_mul_si(a, a, 3);
-    element_add(a, a, cca);
-    element_neg(a, a);
-
-    element_add(b, Zy, Zy);
-
-    element_mul(t0, b, Zy);
-    element_mul(c, a, Zx);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    d_miller_evalfn(e0, a, b, c, Qx, Qy);
-    element_mul(v, v, e0);
+  #define do_tangent() {                  \
+    /* a = -(3 Zx^2 + cc->a) */           \
+    /* b = 2 * Zy */                      \
+    /* c = -(2 Zy^2 + a Zx); */           \
+                                          \
+    element_square(a, Zx);                \
+    element_mul_si(a, a, 3);              \
+    element_add(a, a, cca);               \
+    element_neg(a, a);                    \
+                                          \
+    element_add(b, Zy, Zy);               \
+                                          \
+    element_mul(t0, b, Zy);               \
+    element_mul(c, a, Zx);                \
+    element_add(c, c, t0);                \
+    element_neg(c, c);                    \
+                                          \
+    d_miller_evalfn(e0, a, b, c, Qx, Qy); \
+    element_mul(v, v, e0);                \
   }
 
-  void do_line(void) {
-    // a = -(B.y - A.y) / (B.x - A.x);
-    // b = 1;
-    // c = -(A.y + a * A.x);
-    // but we multiply by B.x - A.x to avoid division.
-
-    element_sub(b, Px, Zx);
-    element_sub(a, Zy, Py);
-    element_mul(t0, b, Zy);
-    element_mul(c, a, Zx);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    d_miller_evalfn(e0, a, b, c, Qx, Qy);
-    element_mul(v, v, e0);
+  #define do_line() {                                     \
+    /* a = -(B.y - A.y) / (B.x - A.x); */                 \
+    /* b = 1; */                                          \
+    /* c = -(A.y + a * A.x); */                           \
+    /* but we multiply by B.x - A.x to avoid division. */ \
+                                                          \
+    element_sub(b, Px, Zx);                               \
+    element_sub(a, Zy, Py);                               \
+    element_mul(t0, b, Zy);                               \
+    element_mul(c, a, Zx);                                \
+    element_add(c, c, t0);                                \
+    element_neg(c, c);                                    \
+                                                          \
+    d_miller_evalfn(e0, a, b, c, Qx, Qy);                 \
+    element_mul(v, v, e0);                                \
   }
 
   element_init(a, Px->field);
@@ -410,6 +415,8 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, element_t P,
   element_clear(c);
   element_clear(t0);
   element_clear(e0);
+  #undef do_tangent
+  #undef do_line
 }
 
 static void (*cc_miller_no_denom_fn)(element_t res, mpz_t q, element_t P,
@@ -495,6 +502,27 @@ static void lucas_even(element_ptr out, element_ptr in, mpz_t cofactor) {
 // The final powering, where we standardize the coset representative.
 static void cc_tatepower(element_ptr out, element_ptr in, pairing_t pairing) {
   pptr p = pairing->data;
+  #define qpower(sign) {                         \
+    polymod_const_mul(e2, inre[1], p->xpowq);    \
+    element_set(e0re, e2);                       \
+    polymod_const_mul(e2, inre[2], p->xpowq2);   \
+    element_add(e0re, e0re, e2);                 \
+    element_add(e0re0, e0re0, inre[0]);          \
+                                                 \
+    if (sign > 0) {                              \
+      polymod_const_mul(e2, inim[1], p->xpowq);  \
+      element_set(e0im, e2);                     \
+      polymod_const_mul(e2, inim[2], p->xpowq2); \
+      element_add(e0im, e0im, e2);               \
+      element_add(e0im0, e0im0, inim[0]);        \
+    } else {                                     \
+      polymod_const_mul(e2, inim[1], p->xpowq);  \
+      element_neg(e0im, e2);                     \
+      polymod_const_mul(e2, inim[2], p->xpowq2); \
+      element_sub(e0im, e0im, e2);               \
+      element_sub(e0im0, e0im0, inim[0]);        \
+    }                                            \
+  }
   if (p->k == 6) {
     // See thesis, section 6.9, "The Final Powering", which gives a formula
     // for the first step of the final powering when Fq6 has been implemented
@@ -510,27 +538,6 @@ static void cc_tatepower(element_ptr out, element_ptr in, pairing_t pairing) {
     element_t *inre = element_x(in)->data;
     element_t *inim = element_y(in)->data;
     // Expressions in the formula are similar, hence the following function.
-    void qpower(int sign) {
-      polymod_const_mul(e2, inre[1], p->xpowq);
-      element_set(e0re, e2);
-      polymod_const_mul(e2, inre[2], p->xpowq2);
-      element_add(e0re, e0re, e2);
-      element_add(e0re0, e0re0, inre[0]);
-
-      if (sign > 0) {
-        polymod_const_mul(e2, inim[1], p->xpowq);
-        element_set(e0im, e2);
-        polymod_const_mul(e2, inim[2], p->xpowq2);
-        element_add(e0im, e0im, e2);
-        element_add(e0im0, e0im0, inim[0]);
-      } else {
-        polymod_const_mul(e2, inim[1], p->xpowq);
-        element_neg(e0im, e2);
-        polymod_const_mul(e2, inim[2], p->xpowq2);
-        element_sub(e0im, e0im, e2);
-        element_sub(e0im0, e0im0, inim[0]);
-      }
-    }
     qpower(1);
     element_set(e3, e0);
     element_set(e0re, element_x(in));
@@ -551,6 +558,7 @@ static void cc_tatepower(element_ptr out, element_ptr in, pairing_t pairing) {
   } else {
     element_pow_mpz(out, in, p->tateexp);
   }
+  #undef qpower
 }
 
 static void cc_finalpow(element_t e) {
@@ -587,7 +595,7 @@ static void cc_millers_no_denom_affine(element_t res, mpz_t q, element_t P[],
   element_t e0;
   const element_ptr cca = curve_a_coeff(P[0]);
   element_ptr Px, Py;
-  element_t* Z=malloc(sizeof(element_t)*n_prod);
+  element_t* Z = pbc_malloc(sizeof(element_t)*n_prod);
   element_ptr Zx, Zy;
 
   /* TODO: when exactly is this not needed?
@@ -598,54 +606,54 @@ static void cc_millers_no_denom_affine(element_t res, mpz_t q, element_t P[],
   }
   */
 
-  void do_tangents(void) {
-    // a = -(3 Zx^2 + cc->a)
-    // b = 2 * Zy
-    // c = -(2 Zy^2 + a Zx);
-    for(i=0; i<n_prod; i++){
-      Px = curve_x_coord(P[i]);
-      Py = curve_y_coord(P[i]);
-      Zx = curve_x_coord(Z[i]);
-      Zy = curve_y_coord(Z[i]);
-
-      element_square(a, Zx);
-      element_mul_si(a, a, 3);
-      element_add(a, a, cca);
-      element_neg(a, a);
-
-      element_add(b, Zy, Zy);
-
-      element_mul(t0, b, Zy);
-      element_mul(c, a, Zx);
-      element_add(c, c, t0);
-      element_neg(c, c);
-
-      d_miller_evalfn(e0, a, b, c, Qx[i], Qy[i]);
-      element_mul(v, v, e0);
-    }
+  #define do_tangents() {                         \
+    /* a = -(3 Zx^2 + cc->a) */                   \
+    /* b = 2 * Zy */                              \
+    /* c = -(2 Zy^2 + a Zx); */                   \
+    for(i=0; i<n_prod; i++){                      \
+      Px = curve_x_coord(P[i]);                   \
+      Py = curve_y_coord(P[i]);                   \
+      Zx = curve_x_coord(Z[i]);                   \
+      Zy = curve_y_coord(Z[i]);                   \
+                                                  \
+      element_square(a, Zx);                      \
+      element_mul_si(a, a, 3);                    \
+      element_add(a, a, cca);                     \
+      element_neg(a, a);                          \
+                                                  \
+      element_add(b, Zy, Zy);                     \
+                                                  \
+      element_mul(t0, b, Zy);                     \
+      element_mul(c, a, Zx);                      \
+      element_add(c, c, t0);                      \
+      element_neg(c, c);                          \
+                                                  \
+      d_miller_evalfn(e0, a, b, c, Qx[i], Qy[i]); \
+      element_mul(v, v, e0);                      \
+    }                                             \
   }
 
-  void do_lines(void) {
-    // a = -(B.y - A.y) / (B.x - A.x);
-    // b = 1;
-    // c = -(A.y + a * A.x);
-    // but we multiply by B.x - A.x to avoid division.
-    for(i=0; i<n_prod; i++){
-      Px = curve_x_coord(P[i]);
-      Py = curve_y_coord(P[i]);
-      Zx = curve_x_coord(Z[i]);
-      Zy = curve_y_coord(Z[i]);
-
-      element_sub(b, Px, Zx);
-      element_sub(a, Zy, Py);
-      element_mul(t0, b, Zy);
-      element_mul(c, a, Zx);
-      element_add(c, c, t0);
-      element_neg(c, c);
-
-      d_miller_evalfn(e0, a, b, c, Qx[i], Qy[i]);
-      element_mul(v, v, e0);
-    }
+  #define do_lines() {                                    \
+    /* a = -(B.y - A.y) / (B.x - A.x); */                 \
+    /* b = 1; */                                          \
+    /* c = -(A.y + a * A.x); */                           \
+    /* but we multiply by B.x - A.x to avoid division. */ \
+    for(i=0; i<n_prod; i++){                              \
+      Px = curve_x_coord(P[i]);                           \
+      Py = curve_y_coord(P[i]);                           \
+      Zx = curve_x_coord(Z[i]);                           \
+      Zy = curve_y_coord(Z[i]);                           \
+                                                          \
+      element_sub(b, Px, Zx);                             \
+      element_sub(a, Zy, Py);                             \
+      element_mul(t0, b, Zy);                             \
+      element_mul(c, a, Zx);                              \
+      element_add(c, c, t0);                              \
+      element_neg(c, c);                                  \
+                                                          \
+      d_miller_evalfn(e0, a, b, c, Qx[i], Qy[i]);         \
+      element_mul(v, v, e0);                              \
+    }                                                     \
   }
 
   Px= curve_x_coord(P[0]); //temporally used to initial a,b, c and etc.
@@ -682,22 +690,24 @@ static void cc_millers_no_denom_affine(element_t res, mpz_t q, element_t P[],
 
   element_clear(v);
   for(i=0; i<n_prod; i++){
-          element_clear(Z[i]);
+    element_clear(Z[i]);
   }
-  free(Z);
+  pbc_free(Z);
   element_clear(a);
   element_clear(b);
   element_clear(c);
   element_clear(t0);
   element_clear(e0);
+  #undef do_tangents
+  #undef do_lines
 }
 
 
 void cc_pairings_affine(element_ptr out, element_t in1[], element_t in2[],
         int n_prod, pairing_t pairing) {
   element_ptr Qbase;
-  element_t* Qx=malloc(sizeof(element_t)*n_prod);
-  element_t* Qy=malloc(sizeof(element_t)*n_prod);
+  element_t* Qx = pbc_malloc(sizeof(element_t)*n_prod);
+  element_t* Qy = pbc_malloc(sizeof(element_t)*n_prod);
   pptr p = pairing->data;
   int i;
   for(i=0; i<n_prod; i++){
@@ -717,9 +727,8 @@ void cc_pairings_affine(element_ptr out, element_t in1[], element_t in2[],
           element_clear(Qx[i]);
                 element_clear(Qy[i]);
   }
-  free(Qx);
-        free(Qy);
-
+  pbc_free(Qx);
+  pbc_free(Qy);
 }
 
 
@@ -795,58 +804,58 @@ static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
   element_ptr Zx;
   element_ptr Zy;
 
-  void store_abc(void) {
-    element_init(pp->a, Fq);
-    element_init(pp->b, Fq);
-    element_init(pp->c, Fq);
-    element_set(pp->a, a);
-    element_set(pp->b, b);
-    element_set(pp->c, c);
-    pp++;
+  #define store_abc() {      \
+    element_init(pp->a, Fq); \
+    element_init(pp->b, Fq); \
+    element_init(pp->c, Fq); \
+    element_set(pp->a, a);   \
+    element_set(pp->b, b);   \
+    element_set(pp->c, c);   \
+    pp++;                    \
   }
 
-  void do_tangent(void) {
-    // a = -slope_tangent(Z.x, Z.y);
-    // b = 1;
-    // c = -(Z.y + a * Z.x);
-    // but we multiply by 2*Z.y to avoid division.
-
-    // a = -Zx * (3 Zx + twicea_2) - a_4;
-    // Common curves: a2 = 0 (and cc->a is a_4), so
-    // a = -(3 Zx^2 + cc->a)
-    // b = 2 * Zy
-    // c = -(2 Zy^2 + a Zx);
-
-    element_square(a, Zx);
-    element_double(t0, a);
-    element_add(a, a, t0);
-    element_add(a, a, cca);
-    element_neg(a, a);
-
-    element_add(b, Zy, Zy);
-
-    element_mul(t0, b, Zy);
-    element_mul(c, a, Zx);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    store_abc();
+  #define do_tangent() {                               \
+    /* a = -slope_tangent(Z.x, Z.y); */                \
+    /* b = 1; */                                       \
+    /* c = -(Z.y + a * Z.x); */                        \
+    /* but we multiply by 2*Z.y to avoid division. */  \
+                                                       \
+    /* a = -Zx * (3 Zx + twicea_2) - a_4; */           \
+    /* Common curves: a2 = 0 (and cc->a is a_4), so */ \
+    /* a = -(3 Zx^2 + cc->a) */                        \
+    /* b = 2 * Zy */                                   \
+    /* c = -(2 Zy^2 + a Zx); */                        \
+                                                       \
+    element_square(a, Zx);                             \
+    element_double(t0, a);                             \
+    element_add(a, a, t0);                             \
+    element_add(a, a, cca);                            \
+    element_neg(a, a);                                 \
+                                                       \
+    element_add(b, Zy, Zy);                            \
+                                                       \
+    element_mul(t0, b, Zy);                            \
+    element_mul(c, a, Zx);                             \
+    element_add(c, c, t0);                             \
+    element_neg(c, c);                                 \
+                                                       \
+    store_abc();                                       \
   }
 
-  void do_line(void) {
-    // a = -(B.y - A.y) / (B.x - A.x);
-    // b = 1;
-    // c = -(A.y + a * A.x);
-    // but we'll multiply by B.x - A.x to avoid division
-
-    element_sub(b, Px, Zx);
-    element_sub(a, Zy, Py);
-    element_mul(t0, b, Zy);
-    element_mul(c, a, Zx);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    store_abc();
+  #define do_line() {                                       \
+    /* a = -(B.y - A.y) / (B.x - A.x); */                   \
+    /* b = 1; */                                            \
+    /* c = -(A.y + a * A.x); */                             \
+    /* but we'll multiply by B.x - A.x to avoid division */ \
+                                                            \
+    element_sub(b, Px, Zx);                                 \
+    element_sub(a, Zy, Py);                                 \
+    element_mul(t0, b, Zy);                                 \
+    element_mul(c, a, Zx);                                  \
+    element_add(c, c, t0);                                  \
+    element_neg(c, c);                                      \
+                                                            \
+    store_abc();                                            \
   }
 
   element_init(Z, P->field);
@@ -882,6 +891,9 @@ static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
   element_clear(b);
   element_clear(c);
   element_clear(Z);
+  #undef store_abc
+  #undef do_tangent
+  #undef do_line
 }
 
 static void d_pairing_pp_clear(pairing_pp_t p) {
@@ -1177,7 +1189,7 @@ static void d_param_init(pbc_param_ptr p) {
 
 // Public interface:
 
-int pbc_param_init_d(pbc_param_ptr par, const char *(*tab)(const char *)) {
+int pbc_param_init_d(pbc_param_ptr par, struct symtab_s *tab) {
   d_param_init(par);
   d_param_ptr p = par->data;
   char s[80];
