@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h> // for intptr_t
 #include <string.h>
 #include <gmp.h>
 #include "pbc_utils.h"
@@ -12,6 +13,7 @@
 #include "pbc_f_param.h"
 #include "pbc_a1_param.h"
 #include "pbc_g_param.h"
+#include "pbc_i_param.h"
 
 #include "misc/symtab.h"
 #include "ecc/param.h"
@@ -42,14 +44,7 @@ static const char *token_get(token_t tok, const char *input, const char *end) {
   int n = 32;
   int i;
   char c;
-  int get(void) {
-    if (!end || input < end) {
-      c = *input++;
-      return !c;
-    } else {
-      return 1;
-    }
-  }
+  #define get() (((!end || input < end) && *input) ? (c = *input++, 0) : 1)
   // Skip whitespace and comments.
   for(;;) {
     do {
@@ -84,6 +79,7 @@ static const char *token_get(token_t tok, const char *input, const char *end) {
   buf[i] = 0;
   tok->s = buf;
   return input;
+  #undef get
 }
 
 static void token_init(token_t tok) {
@@ -97,21 +93,19 @@ static void token_clear(token_t tok) {
 
 static void read_symtab(symtab_t tab, const char *input, size_t limit) {
   token_t tok;
-  char *s, *s1;
   const char *inputend = limit ? input + limit : NULL;
   token_init(tok);
   for (;;) {
     input = token_get(tok, input, inputend);
     if (tok->type != token_word) break;
-    s = pbc_strdup(tok->s);
+    char *key = pbc_strdup(tok->s);
     input = token_get(tok, input, inputend);
     if (tok->type != token_word) {
-      pbc_free(s);
+      pbc_free(key);
       break;
     }
-    s1 = pbc_strdup(tok->s);
-    symtab_put(tab, s1, s);
-    pbc_free(s);
+    symtab_put(tab, pbc_strdup(tok->s), key);
+    pbc_free(key);
   }
   token_clear(tok);
 }
@@ -137,8 +131,16 @@ void param_out_int(FILE *stream, char *s, int i) {
   mpz_clear(z);
 }
 
-int lookup_mpz(mpz_t z, const char *(*tab)(const char *), const char *key) {
-  const char *data = tab(key);
+static const char *lookup(symtab_t tab, const char *key) {
+  if (!symtab_has(tab, key)) {
+    pbc_error("missing param: `%s'", key);
+    return NULL;
+  }
+  return symtab_at(tab, key);
+}
+
+int lookup_mpz(mpz_t z, symtab_t tab, const char *key) {
+  const char *data = lookup(tab, key);
   if (!data) {
     pbc_error("missing param: `%s'", key);
     return 1;
@@ -147,9 +149,9 @@ int lookup_mpz(mpz_t z, const char *(*tab)(const char *), const char *key) {
   return 0;
 }
 
-int lookup_int(int *n, const char *(*tab)(const char *), const char *key) {
+int lookup_int(int *n, symtab_t tab, const char *key) {
   mpz_t z;
-  const char *data = tab(key);
+  const char *data = lookup(tab, key);
   if (!data) {
     pbc_error("missing param: `%s'", key);
     return 1;
@@ -164,18 +166,11 @@ int lookup_int(int *n, const char *(*tab)(const char *), const char *key) {
 }
 
 static int param_set_tab(pbc_param_t par, symtab_t tab) {
-  const char *lookup(const char *key) {
-    if (!symtab_has(tab, key)) {
-      pbc_error("missing param: `%s'", key);
-      return NULL;
-    }
-    return symtab_at(tab, key);
-  }
-  const char *s = lookup("type");
+  const char *s = lookup(tab, "type");
 
   static struct {
     char *s;
-    int (*fun)(pbc_param_ptr, const char *(*)(const char *));
+    int (*fun)(pbc_param_ptr, symtab_t tab);
   } funtab[] = {
       { "a", pbc_param_init_a },
       { "d", pbc_param_init_d },
@@ -183,6 +178,7 @@ static int param_set_tab(pbc_param_t par, symtab_t tab) {
       { "f", pbc_param_init_f },
       { "g", pbc_param_init_g },
       { "a1", pbc_param_init_a1 },
+      { "i", pbc_param_init_i },
   };
 
   int res = 1;
@@ -190,7 +186,7 @@ static int param_set_tab(pbc_param_t par, symtab_t tab) {
     unsigned int i;
     for(i = 0; i < sizeof(funtab)/sizeof(*funtab); i++) {
       if (!strcmp(s, funtab[i].s)) {
-        res = funtab[i].fun(par, lookup);
+        res = funtab[i].fun(par, tab);
         if (res) pbc_error("bad pairing parameters");
         return res;
       }
