@@ -10,20 +10,21 @@
 // e.g. $ hilbertpoly 3 1000000
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdint.h> // for intptr_t
 #include <stdlib.h> //for atoi, exit
 #include <gmp.h>
 #include "pbc_utils.h"
 #include "pbc_hilbert.h"
 
-int main(int argc, char **argv) {
-  void xpow(int degree) {
-    if (degree == 1) {
-      printf("X");
-    } else if (degree) {
-      printf("X^%d", degree);
-    }
+static void xpow(int degree) {
+  if (degree == 1) {
+    printf("X");
+  } else if (degree) {
+    printf("X^%d", degree);
   }
+}
 
+int main(int argc, char **argv) {
   int D      = argc > 1 ? atoi(argv[1]) : 3;
   if (D <= 0) pbc_die("D must be positive.");
 
