@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.5.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton implementation for Bison's Yacc-like parsers in C
+/* Bison implementation for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.4.1"
+#define YYBISON_VERSION "2.5"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -67,18 +65,19 @@
 
 /* Copy the first part of user declarations.  */
 
-/* Line 189 of yacc.c  */
+/* Line 268 of yacc.c  */
 #line 1 "pbc/parser.y"
 
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdint.h> // for intptr_t
 #include <gmp.h>
 #include "pbc_utils.h"
 #include "pbc_field.h"
 
 #include "pbc_tree.h"
 #define YYSTYPE tree_ptr
-void yyerror(char *s);
+void yyerror(const char *s);
 int yylex(void);
 
 #define YY_NO_INPUT
@@ -87,8 +86,8 @@ int yylex(void);
 extern int option_easy;
 
 
-/* Line 189 of yacc.c  */
-#line 92 "pbc/parser.tab.c"
+/* Line 268 of yacc.c  */
+#line 91 "pbc/parser.tab.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -159,8 +158,8 @@ typedef int YYSTYPE;
 /* Copy the second part of user declarations.  */
 
 
-/* Line 264 of yacc.c  */
-#line 164 "pbc/parser.tab.c"
+/* Line 343 of yacc.c  */
+#line 163 "pbc/parser.tab.c"
 
 #ifdef short
 # undef short
@@ -210,7 +209,7 @@ typedef short int yytype_int16;
 #define YYSIZE_MAXIMUM ((YYSIZE_T) -1)
 
 #ifndef YY_
-# if YYENABLE_NLS
+# if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
 #   define YY_(msgid) dgettext ("bison-runtime", msgid)
@@ -263,11 +262,11 @@ YYID (yyi)
 #    define alloca _alloca
 #   else
 #    define YYSTACK_ALLOC alloca
-#    if ! defined _ALLOCA_H && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
+#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 #     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-#     ifndef _STDLIB_H
-#      define _STDLIB_H 1
+#     ifndef EXIT_SUCCESS
+#      define EXIT_SUCCESS 0
 #     endif
 #    endif
 #   endif
@@ -290,24 +289,24 @@ YYID (yyi)
 #  ifndef YYSTACK_ALLOC_MAXIMUM
 #   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
 #  endif
-#  if (defined __cplusplus && ! defined _STDLIB_H \
+#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
        && ! ((defined YYMALLOC || defined malloc) \
 	     && (defined YYFREE || defined free)))
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-#   ifndef _STDLIB_H
-#    define _STDLIB_H 1
+#   ifndef EXIT_SUCCESS
+#    define EXIT_SUCCESS 0
 #   endif
 #  endif
 #  ifndef YYMALLOC
 #   define YYMALLOC malloc
-#   if ! defined malloc && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
+#   if ! defined malloc && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
 #   endif
 #  endif
 #  ifndef YYFREE
 #   define YYFREE free
-#   if ! defined free && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
+#   if ! defined free && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 void free (void *); /* INFRINGES ON USER NAME SPACE */
 #   endif
@@ -336,23 +335,7 @@ union yyalloc
      ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
       + YYSTACK_GAP_MAXIMUM)
 
-/* Copy COUNT objects from FROM to TO.  The source and destination do
-   not overlap.  */
-# ifndef YYCOPY
-#  if defined __GNUC__ && 1 < __GNUC__
-#   define YYCOPY(To, From, Count) \
-      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
-#  else
-#   define YYCOPY(To, From, Count)		\
-      do					\
-	{					\
-	  YYSIZE_T yyi;				\
-	  for (yyi = 0; yyi < (Count); yyi++)	\
-	    (To)[yyi] = (From)[yyi];		\
-	}					\
-      while (YYID (0))
-#  endif
-# endif
+# define YYCOPY_NEEDED 1
 
 /* Relocate STACK from its old location to the new one.  The
    local variables YYSIZE and YYSTACKSIZE give the old and new number of
@@ -372,6 +355,26 @@ union yyalloc
 
 #endif
 
+#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
+/* Copy COUNT objects from FROM to TO.  The source and destination do
+   not overlap.  */
+# ifndef YYCOPY
+#  if defined __GNUC__ && 1 < __GNUC__
+#   define YYCOPY(To, From, Count) \
+      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
+#  else
+#   define YYCOPY(To, From, Count)		\
+      do					\
+	{					\
+	  YYSIZE_T yyi;				\
+	  for (yyi = 0; yyi < (Count); yyi++)	\
+	    (To)[yyi] = (From)[yyi];		\
+	}					\
+      while (YYID (0))
+#  endif
+# endif
+#endif /* !YYCOPY_NEEDED */
+
 /* YYFINAL -- State number of the termination state.  */
 #define YYFINAL  2
 /* YYLAST -- Last index in YYTABLE.  */
@@ -461,11 +464,11 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    34,    34,    36,    40,    41,    47,    48,    52,    53,
-      57,    58,    62,    63,    64,    65,    66,    67,    68,    69,
-      70,    71,    72,    73,    74,    75,    76,    77,    78,    83,
-      84,    85,    89,    90,    94,    95,    99,   100,   104,   108,
-     109
+       0,    35,    35,    37,    41,    42,    48,    49,    53,    54,
+      58,    59,    63,    64,    65,    66,    67,    68,    69,    70,
+      71,    72,    73,    74,    75,    76,    77,    78,    79,    84,
+      85,    86,    90,    91,    95,    96,   100,   101,   105,   109,
+     110
 };
 #endif
 
@@ -515,8 +518,8 @@ static const yytype_uint8 yyr2[] =
        3
 };
 
-/* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
-   STATE-NUM when YYTABLE doesn't specify something else to do.  Zero
+/* YYDEFACT[STATE-NAME] -- Default reduction number in state STATE-NUM.
+   Performed when YYTABLE doesn't specify something else to do.  Zero
    means the default is an error.  */
 static const yytype_uint8 yydefact[] =
 {
@@ -561,8 +564,7 @@ static const yytype_int8 yypgoto[] =
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
    positive, shift that token.  If negative, reduce the rule which
-   number is the opposite.  If zero, do what YYDEFACT says.
-   If YYTABLE_NINF, syntax error.  */
+   number is the opposite.  If YYTABLE_NINF, syntax error.  */
 #define YYTABLE_NINF -1
 static const yytype_uint8 yytable[] =
 {
@@ -583,6 +585,12 @@ static const yytype_uint8 yytable[] =
       26,    27,     0,    28,    29,    30,    31,     0,    32
 };
 
+#define yypact_value_is_default(yystate) \
+  ((yystate) == (-21))
+
+#define yytable_value_is_error(yytable_value) \
+  YYID (0)
+
 static const yytype_int8 yycheck[] =
 {
        6,     7,     8,    23,    24,    25,    26,     6,    28,    15,
@@ -628,9 +636,18 @@ static const yytype_uint8 yystos[] =
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
    to ease the transition to the new meaning of YYERROR, for GCC.
-   Once GCC version 2 has supplanted version 1, this can go.  */
+   Once GCC version 2 has supplanted version 1, this can go.  However,
+   YYFAIL appears to be in use.  Nevertheless, it is formally deprecated
+   in Bison 2.4.2's NEWS entry, where a plan to phase it out is
+   discussed.  */
 
 #define YYFAIL		goto yyerrlab
+#if defined YYFAIL
+  /* This is here to suppress warnings from the GCC cpp's
+     -Wunused-macros.  Normally we don't worry about that warning, but
+     some users do, and we want to make it easy for users to remove
+     YYFAIL uses, which will produce warnings from Bison 2.5.  */
+#endif
 
 #define YYRECOVERING()  (!!yyerrstatus)
 
@@ -640,7 +657,6 @@ do								\
     {								\
       yychar = (Token);						\
       yylval = (Value);						\
-      yytoken = YYTRANSLATE (yychar);				\
       YYPOPSTACK (1);						\
       goto yybackup;						\
     }								\
@@ -682,19 +698,10 @@ while (YYID (0))
 #endif
 
 
-/* YY_LOCATION_PRINT -- Print the location on the stream.
-   This macro was not mandated originally: define only if we know
-   we won't break user code: when these are the locations we know.  */
+/* This macro is provided for backward compatibility. */
 
 #ifndef YY_LOCATION_PRINT
-# if YYLTYPE_IS_TRIVIAL
-#  define YY_LOCATION_PRINT(File, Loc)			\
-     fprintf (File, "%d.%d-%d.%d",			\
-	      (Loc).first_line, (Loc).first_column,	\
-	      (Loc).last_line,  (Loc).last_column)
-# else
-#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
-# endif
+# define YY_LOCATION_PRINT(File, Loc) ((void) 0)
 #endif
 
 
@@ -886,7 +893,6 @@ int yydebug;
 # define YYMAXDEPTH 10000
 #endif
 
-
 
 #if YYERROR_VERBOSE
 
@@ -989,115 +995,142 @@ yytnamerr (char *yyres, const char *yystr)
 }
 # endif
 
-/* Copy into YYRESULT an error message about the unexpected token
-   YYCHAR while in state YYSTATE.  Return the number of bytes copied,
-   including the terminating null byte.  If YYRESULT is null, do not
-   copy anything; just return the number of bytes that would be
-   copied.  As a special case, return 0 if an ordinary "syntax error"
-   message will do.  Return YYSIZE_MAXIMUM if overflow occurs during
-   size calculation.  */
-static YYSIZE_T
-yysyntax_error (char *yyresult, int yystate, int yychar)
+/* Copy into *YYMSG, which is of size *YYMSG_ALLOC, an error message
+   about the unexpected token YYTOKEN for the state stack whose top is
+   YYSSP.
+
+   Return 0 if *YYMSG was successfully written.  Return 1 if *YYMSG is
+   not large enough to hold the message.  In that case, also set
+   *YYMSG_ALLOC to the required number of bytes.  Return 2 if the
+   required number of bytes is too large to store.  */
+static int
+yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
+                yytype_int16 *yyssp, int yytoken)
 {
-  int yyn = yypact[yystate];
+  YYSIZE_T yysize0 = yytnamerr (0, yytname[yytoken]);
+  YYSIZE_T yysize = yysize0;
+  YYSIZE_T yysize1;
+  enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
+  /* Internationalized format string. */
+  const char *yyformat = 0;
+  /* Arguments of yyformat. */
+  char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
+  /* Number of reported tokens (one for the "unexpected", one per
+     "expected"). */
+  int yycount = 0;
 
-  if (! (YYPACT_NINF < yyn && yyn <= YYLAST))
-    return 0;
-  else
+  /* There are many possibilities here to consider:
+     - Assume YYFAIL is not used.  It's too flawed to consider.  See
+       <http://lists.gnu.org/archive/html/bison-patches/2009-12/msg00024.html>
+       for details.  YYERROR is fine as it does not invoke this
+       function.
+     - If this state is a consistent state with a default action, then
+       the only way this function was invoked is if the default action
+       is an error action.  In that case, don't check for expected
+       tokens because there are none.
+     - The only way there can be no lookahead present (in yychar) is if
+       this state is a consistent state with a default action.  Thus,
+       detecting the absence of a lookahead is sufficient to determine
+       that there is no unexpected or expected token to report.  In that
+       case, just report a simple "syntax error".
+     - Don't assume there isn't a lookahead just because this state is a
+       consistent state with a default action.  There might have been a
+       previous inconsistent state, consistent state with a non-default
+       action, or user semantic action that manipulated yychar.
+     - Of course, the expected token list depends on states to have
+       correct lookahead information, and it depends on the parser not
+       to perform extra reductions after fetching a lookahead from the
+       scanner and before detecting a syntax error.  Thus, state merging
+       (from LALR or IELR) and default reductions corrupt the expected
+       token list.  However, the list is correct for canonical LR with
+       one exception: it will still contain any token that will not be
+       accepted due to an error action in a later state.
+  */
+  if (yytoken != YYEMPTY)
     {
-      int yytype = YYTRANSLATE (yychar);
-      YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
-      YYSIZE_T yysize = yysize0;
-      YYSIZE_T yysize1;
-      int yysize_overflow = 0;
-      enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
-      char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
-      int yyx;
+      int yyn = yypact[*yyssp];
+      yyarg[yycount++] = yytname[yytoken];
+      if (!yypact_value_is_default (yyn))
+        {
+          /* Start YYX at -YYN if negative to avoid negative indexes in
+             YYCHECK.  In other words, skip the first -YYN actions for
+             this state because they are default actions.  */
+          int yyxbegin = yyn < 0 ? -yyn : 0;
+          /* Stay within bounds of both yycheck and yytname.  */
+          int yychecklim = YYLAST - yyn + 1;
+          int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
+          int yyx;
 
-# if 0
-      /* This is so xgettext sees the translatable formats that are
-	 constructed on the fly.  */
-      YY_("syntax error, unexpected %s");
-      YY_("syntax error, unexpected %s, expecting %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s or %s");
-      YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
-# endif
-      char *yyfmt;
-      char const *yyf;
-      static char const yyunexpected[] = "syntax error, unexpected %s";
-      static char const yyexpecting[] = ", expecting %s";
-      static char const yyor[] = " or %s";
-      char yyformat[sizeof yyunexpected
-		    + sizeof yyexpecting - 1
-		    + ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
-		       * (sizeof yyor - 1))];
-      char const *yyprefix = yyexpecting;
-
-      /* Start YYX at -YYN if negative to avoid negative indexes in
-	 YYCHECK.  */
-      int yyxbegin = yyn < 0 ? -yyn : 0;
-
-      /* Stay within bounds of both yycheck and yytname.  */
-      int yychecklim = YYLAST - yyn + 1;
-      int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
-      int yycount = 1;
-
-      yyarg[0] = yytname[yytype];
-      yyfmt = yystpcpy (yyformat, yyunexpected);
-
-      for (yyx = yyxbegin; yyx < yyxend; ++yyx)
-	if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	  {
-	    if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
-	      {
-		yycount = 1;
-		yysize = yysize0;
-		yyformat[sizeof yyunexpected - 1] = '\0';
-		break;
-	      }
-	    yyarg[yycount++] = yytname[yyx];
-	    yysize1 = yysize + yytnamerr (0, yytname[yyx]);
-	    yysize_overflow |= (yysize1 < yysize);
-	    yysize = yysize1;
-	    yyfmt = yystpcpy (yyfmt, yyprefix);
-	    yyprefix = yyor;
-	  }
-
-      yyf = YY_(yyformat);
-      yysize1 = yysize + yystrlen (yyf);
-      yysize_overflow |= (yysize1 < yysize);
-      yysize = yysize1;
-
-      if (yysize_overflow)
-	return YYSIZE_MAXIMUM;
-
-      if (yyresult)
-	{
-	  /* Avoid sprintf, as that infringes on the user's name space.
-	     Don't have undefined behavior even if the translation
-	     produced a string with the wrong number of "%s"s.  */
-	  char *yyp = yyresult;
-	  int yyi = 0;
-	  while ((*yyp = *yyf) != '\0')
-	    {
-	      if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
-		{
-		  yyp += yytnamerr (yyp, yyarg[yyi++]);
-		  yyf += 2;
-		}
-	      else
-		{
-		  yyp++;
-		  yyf++;
-		}
-	    }
-	}
-      return yysize;
+          for (yyx = yyxbegin; yyx < yyxend; ++yyx)
+            if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR
+                && !yytable_value_is_error (yytable[yyx + yyn]))
+              {
+                if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
+                  {
+                    yycount = 1;
+                    yysize = yysize0;
+                    break;
+                  }
+                yyarg[yycount++] = yytname[yyx];
+                yysize1 = yysize + yytnamerr (0, yytname[yyx]);
+                if (! (yysize <= yysize1
+                       && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                  return 2;
+                yysize = yysize1;
+              }
+        }
     }
+
+  switch (yycount)
+    {
+# define YYCASE_(N, S)                      \
+      case N:                               \
+        yyformat = S;                       \
+      break
+      YYCASE_(0, YY_("syntax error"));
+      YYCASE_(1, YY_("syntax error, unexpected %s"));
+      YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
+      YYCASE_(3, YY_("syntax error, unexpected %s, expecting %s or %s"));
+      YYCASE_(4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
+      YYCASE_(5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
+# undef YYCASE_
+    }
+
+  yysize1 = yysize + yystrlen (yyformat);
+  if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+    return 2;
+  yysize = yysize1;
+
+  if (*yymsg_alloc < yysize)
+    {
+      *yymsg_alloc = 2 * yysize;
+      if (! (yysize <= *yymsg_alloc
+             && *yymsg_alloc <= YYSTACK_ALLOC_MAXIMUM))
+        *yymsg_alloc = YYSTACK_ALLOC_MAXIMUM;
+      return 1;
+    }
+
+  /* Avoid sprintf, as that infringes on the user's name space.
+     Don't have undefined behavior even if the translation
+     produced a string with the wrong number of "%s"s.  */
+  {
+    char *yyp = *yymsg;
+    int yyi = 0;
+    while ((*yyp = *yyformat) != '\0')
+      if (*yyp == '%' && yyformat[1] == 's' && yyi < yycount)
+        {
+          yyp += yytnamerr (yyp, yyarg[yyi++]);
+          yyformat += 2;
+        }
+      else
+        {
+          yyp++;
+          yyformat++;
+        }
+  }
+  return 0;
 }
 #endif /* YYERROR_VERBOSE */
-
 
 /*-----------------------------------------------.
 | Release the memory associated to this symbol.  |
@@ -1130,6 +1163,7 @@ yydestruct (yymsg, yytype, yyvaluep)
     }
 }
 
+
 /* Prevent warnings from -Wmissing-prototypes.  */
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
@@ -1156,10 +1190,9 @@ YYSTYPE yylval;
 int yynerrs;
 
 
-
-/*-------------------------.
-| yyparse or yypush_parse.  |
-`-------------------------*/
+/*----------.
+| yyparse.  |
+`----------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1183,8 +1216,6 @@ yyparse ()
 #endif
 #endif
 {
-
-
     int yystate;
     /* Number of tokens to shift before error messages enabled.  */
     int yyerrstatus;
@@ -1339,7 +1370,7 @@ yybackup:
 
   /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
-  if (yyn == YYPACT_NINF)
+  if (yypact_value_is_default (yyn))
     goto yydefault;
 
   /* Not known => get a lookahead token if don't already have one.  */
@@ -1370,8 +1401,8 @@ yybackup:
   yyn = yytable[yyn];
   if (yyn <= 0)
     {
-      if (yyn == 0 || yyn == YYTABLE_NINF)
-	goto yyerrlab;
+      if (yytable_value_is_error (yyn))
+        goto yyerrlab;
       yyn = -yyn;
       goto yyreduce;
     }
@@ -1426,222 +1457,233 @@ yyreduce:
     {
         case 3:
 
-/* Line 1455 of yacc.c  */
-#line 36 "pbc/parser.y"
-    { tree_eval_stmt((yyvsp[(2) - (2)])); ;}
+/* Line 1806 of yacc.c  */
+#line 37 "pbc/parser.y"
+    { tree_eval_stmt((yyvsp[(2) - (2)])); }
     break;
 
   case 5:
 
-/* Line 1455 of yacc.c  */
-#line 41 "pbc/parser.y"
+/* Line 1806 of yacc.c  */
+#line 42 "pbc/parser.y"
     {
       (yyval) = tree_new_define((yyvsp[(2) - (8)]), (yyvsp[(4) - (8)]), (yyvsp[(7) - (8)]));
-    ;}
+    }
     break;
 
   case 6:
 
-/* Line 1455 of yacc.c  */
-#line 47 "pbc/parser.y"
-    { (yyval) = tree_new_empty_stmt_list(); ;}
+/* Line 1806 of yacc.c  */
+#line 48 "pbc/parser.y"
+    { (yyval) = tree_new_empty_stmt_list(); }
     break;
 
   case 7:
 
-/* Line 1455 of yacc.c  */
-#line 48 "pbc/parser.y"
-    { tree_append((yyvsp[(1) - (2)]), (yyvsp[(2) - (2)])); ;}
+/* Line 1806 of yacc.c  */
+#line 49 "pbc/parser.y"
+    { tree_append((yyvsp[(1) - (2)]), (yyvsp[(2) - (2)])); }
     break;
 
   case 8:
 
-/* Line 1455 of yacc.c  */
-#line 52 "pbc/parser.y"
-    { (yyval) = tree_new_empty_parms(); ;}
+/* Line 1806 of yacc.c  */
+#line 53 "pbc/parser.y"
+    { (yyval) = tree_new_empty_parms(); }
     break;
 
   case 10:
 
-/* Line 1455 of yacc.c  */
-#line 57 "pbc/parser.y"
-    { (yyval) = tree_new_empty_parms(); tree_append((yyval), (yyvsp[(1) - (1)])); ;}
+/* Line 1806 of yacc.c  */
+#line 58 "pbc/parser.y"
+    { (yyval) = tree_new_empty_parms(); tree_append((yyval), (yyvsp[(1) - (1)])); }
     break;
 
   case 11:
 
-/* Line 1455 of yacc.c  */
-#line 58 "pbc/parser.y"
-    { tree_append((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 59 "pbc/parser.y"
+    { tree_append((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 13:
 
-/* Line 1455 of yacc.c  */
-#line 63 "pbc/parser.y"
-    { (yyval) = tree_new_assign((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 64 "pbc/parser.y"
+    { (yyval) = tree_new_assign((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 14:
 
-/* Line 1455 of yacc.c  */
-#line 64 "pbc/parser.y"
-    { (yyval) = tree_new_ternary((yyvsp[(1) - (5)]), (yyvsp[(3) - (5)]), (yyvsp[(5) - (5)])); ;}
+/* Line 1806 of yacc.c  */
+#line 65 "pbc/parser.y"
+    { (yyval) = tree_new_ternary((yyvsp[(1) - (5)]), (yyvsp[(3) - (5)]), (yyvsp[(5) - (5)])); }
     break;
 
   case 16:
 
-/* Line 1455 of yacc.c  */
-#line 66 "pbc/parser.y"
-    { (yyval) = tree_new_item((yyvsp[(1) - (4)]), (yyvsp[(3) - (4)])); ;}
+/* Line 1806 of yacc.c  */
+#line 67 "pbc/parser.y"
+    { (yyval) = tree_new_item((yyvsp[(1) - (4)]), (yyvsp[(3) - (4)])); }
     break;
 
   case 17:
 
-/* Line 1455 of yacc.c  */
-#line 67 "pbc/parser.y"
-    { (yyval) = tree_new_eq((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 68 "pbc/parser.y"
+    { (yyval) = tree_new_eq((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 18:
 
-/* Line 1455 of yacc.c  */
-#line 68 "pbc/parser.y"
-    { (yyval) = tree_new_ne((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 69 "pbc/parser.y"
+    { (yyval) = tree_new_ne((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 19:
 
-/* Line 1455 of yacc.c  */
-#line 69 "pbc/parser.y"
-    { (yyval) = tree_new_le((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 70 "pbc/parser.y"
+    { (yyval) = tree_new_le((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 20:
 
-/* Line 1455 of yacc.c  */
-#line 70 "pbc/parser.y"
-    { (yyval) = tree_new_ge((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 71 "pbc/parser.y"
+    { (yyval) = tree_new_ge((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 21:
 
-/* Line 1455 of yacc.c  */
-#line 71 "pbc/parser.y"
-    { (yyval) = tree_new_lt((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 72 "pbc/parser.y"
+    { (yyval) = tree_new_lt((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 22:
 
-/* Line 1455 of yacc.c  */
-#line 72 "pbc/parser.y"
-    { (yyval) = tree_new_gt((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 73 "pbc/parser.y"
+    { (yyval) = tree_new_gt((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 23:
 
-/* Line 1455 of yacc.c  */
-#line 73 "pbc/parser.y"
-    { (yyval) = tree_new_add((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 74 "pbc/parser.y"
+    { (yyval) = tree_new_add((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 24:
 
-/* Line 1455 of yacc.c  */
-#line 74 "pbc/parser.y"
-    { (yyval) = tree_new_sub((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 75 "pbc/parser.y"
+    { (yyval) = tree_new_sub((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 25:
 
-/* Line 1455 of yacc.c  */
-#line 75 "pbc/parser.y"
-    { (yyval) = tree_new_mul((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 76 "pbc/parser.y"
+    { (yyval) = tree_new_mul((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 26:
 
-/* Line 1455 of yacc.c  */
-#line 76 "pbc/parser.y"
-    { (yyval) = tree_new_div((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 77 "pbc/parser.y"
+    { (yyval) = tree_new_div((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 27:
 
-/* Line 1455 of yacc.c  */
-#line 77 "pbc/parser.y"
-    { (yyval) = tree_new_pow((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 78 "pbc/parser.y"
+    { (yyval) = tree_new_pow((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 28:
 
-/* Line 1455 of yacc.c  */
-#line 78 "pbc/parser.y"
-    { (yyval) = tree_new_neg((yyvsp[(2) - (2)])); ;}
+/* Line 1806 of yacc.c  */
+#line 79 "pbc/parser.y"
+    { (yyval) = tree_new_neg((yyvsp[(2) - (2)])); }
     break;
 
   case 29:
 
-/* Line 1455 of yacc.c  */
-#line 83 "pbc/parser.y"
-    { (yyval) = (yyvsp[(3) - (4)]); tree_set_fun((yyval), (yyvsp[(1) - (4)])); ;}
+/* Line 1806 of yacc.c  */
+#line 84 "pbc/parser.y"
+    { (yyval) = (yyvsp[(3) - (4)]); tree_set_fun((yyval), (yyvsp[(1) - (4)])); }
     break;
 
   case 30:
 
-/* Line 1455 of yacc.c  */
-#line 84 "pbc/parser.y"
-    { (yyval) = (yyvsp[(2) - (3)]) ;}
+/* Line 1806 of yacc.c  */
+#line 85 "pbc/parser.y"
+    { (yyval) = (yyvsp[(2) - (3)]); }
     break;
 
   case 32:
 
-/* Line 1455 of yacc.c  */
-#line 89 "pbc/parser.y"
-    { (yyval) = tree_new_funcall(); ;}
+/* Line 1806 of yacc.c  */
+#line 90 "pbc/parser.y"
+    { (yyval) = tree_new_funcall(); }
     break;
 
   case 34:
 
-/* Line 1455 of yacc.c  */
-#line 94 "pbc/parser.y"
-    { tree_append((yyval) = tree_new_funcall(), (yyvsp[(1) - (1)])); ;}
+/* Line 1806 of yacc.c  */
+#line 95 "pbc/parser.y"
+    { tree_append((yyval) = tree_new_funcall(), (yyvsp[(1) - (1)])); }
     break;
 
   case 35:
 
-/* Line 1455 of yacc.c  */
-#line 95 "pbc/parser.y"
-    { tree_append((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 96 "pbc/parser.y"
+    { tree_append((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
   case 38:
 
-/* Line 1455 of yacc.c  */
-#line 104 "pbc/parser.y"
-    { (yyval) = (yyvsp[(2) - (3)]); ;}
+/* Line 1806 of yacc.c  */
+#line 105 "pbc/parser.y"
+    { (yyval) = (yyvsp[(2) - (3)]); }
     break;
 
   case 39:
 
-/* Line 1455 of yacc.c  */
-#line 108 "pbc/parser.y"
-    { (yyval) = tree_new_list((yyvsp[(1) - (1)])); ;}
+/* Line 1806 of yacc.c  */
+#line 109 "pbc/parser.y"
+    { (yyval) = tree_new_list((yyvsp[(1) - (1)])); }
     break;
 
   case 40:
 
-/* Line 1455 of yacc.c  */
-#line 109 "pbc/parser.y"
-    { tree_append((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); ;}
+/* Line 1806 of yacc.c  */
+#line 110 "pbc/parser.y"
+    { tree_append((yyvsp[(1) - (3)]), (yyvsp[(3) - (3)])); }
     break;
 
 
 
-/* Line 1455 of yacc.c  */
-#line 1643 "pbc/parser.tab.c"
+/* Line 1806 of yacc.c  */
+#line 1674 "pbc/parser.tab.c"
       default: break;
     }
+  /* User semantic actions sometimes alter yychar, and that requires
+     that yytoken be updated with the new translation.  We take the
+     approach of translating immediately before every use of yytoken.
+     One alternative is translating here after every semantic action,
+     but that translation would be missed if the semantic action invokes
+     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
+     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
+     incorrect destructor might then be invoked immediately.  In the
+     case of YYERROR or YYBACKUP, subsequent parser actions might lead
+     to an incorrect destructor call or verbose syntax error message
+     before the lookahead is translated.  */
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
 
   YYPOPSTACK (yylen);
@@ -1669,6 +1711,10 @@ yyreduce:
 | yyerrlab -- here on detecting error |
 `------------------------------------*/
 yyerrlab:
+  /* Make sure we have latest lookahead translation.  See comments at
+     user semantic actions for why this is necessary.  */
+  yytoken = yychar == YYEMPTY ? YYEMPTY : YYTRANSLATE (yychar);
+
   /* If not already recovering from an error, report this error.  */
   if (!yyerrstatus)
     {
@@ -1676,37 +1722,36 @@ yyerrlab:
 #if ! YYERROR_VERBOSE
       yyerror (YY_("syntax error"));
 #else
+# define YYSYNTAX_ERROR yysyntax_error (&yymsg_alloc, &yymsg, \
+                                        yyssp, yytoken)
       {
-	YYSIZE_T yysize = yysyntax_error (0, yystate, yychar);
-	if (yymsg_alloc < yysize && yymsg_alloc < YYSTACK_ALLOC_MAXIMUM)
-	  {
-	    YYSIZE_T yyalloc = 2 * yysize;
-	    if (! (yysize <= yyalloc && yyalloc <= YYSTACK_ALLOC_MAXIMUM))
-	      yyalloc = YYSTACK_ALLOC_MAXIMUM;
-	    if (yymsg != yymsgbuf)
-	      YYSTACK_FREE (yymsg);
-	    yymsg = (char *) YYSTACK_ALLOC (yyalloc);
-	    if (yymsg)
-	      yymsg_alloc = yyalloc;
-	    else
-	      {
-		yymsg = yymsgbuf;
-		yymsg_alloc = sizeof yymsgbuf;
-	      }
-	  }
-
-	if (0 < yysize && yysize <= yymsg_alloc)
-	  {
-	    (void) yysyntax_error (yymsg, yystate, yychar);
-	    yyerror (yymsg);
-	  }
-	else
-	  {
-	    yyerror (YY_("syntax error"));
-	    if (yysize != 0)
-	      goto yyexhaustedlab;
-	  }
+        char const *yymsgp = YY_("syntax error");
+        int yysyntax_error_status;
+        yysyntax_error_status = YYSYNTAX_ERROR;
+        if (yysyntax_error_status == 0)
+          yymsgp = yymsg;
+        else if (yysyntax_error_status == 1)
+          {
+            if (yymsg != yymsgbuf)
+              YYSTACK_FREE (yymsg);
+            yymsg = (char *) YYSTACK_ALLOC (yymsg_alloc);
+            if (!yymsg)
+              {
+                yymsg = yymsgbuf;
+                yymsg_alloc = sizeof yymsgbuf;
+                yysyntax_error_status = 2;
+              }
+            else
+              {
+                yysyntax_error_status = YYSYNTAX_ERROR;
+                yymsgp = yymsg;
+              }
+          }
+        yyerror (yymsgp);
+        if (yysyntax_error_status == 2)
+          goto yyexhaustedlab;
       }
+# undef YYSYNTAX_ERROR
 #endif
     }
 
@@ -1765,7 +1810,7 @@ yyerrlab1:
   for (;;)
     {
       yyn = yypact[yystate];
-      if (yyn != YYPACT_NINF)
+      if (!yypact_value_is_default (yyn))
 	{
 	  yyn += YYTERROR;
 	  if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYTERROR)
@@ -1824,8 +1869,13 @@ yyexhaustedlab:
 
 yyreturn:
   if (yychar != YYEMPTY)
-     yydestruct ("Cleanup: discarding lookahead",
-		 yytoken, &yylval);
+    {
+      /* Make sure we have latest lookahead translation.  See comments at
+         user semantic actions for why this is necessary.  */
+      yytoken = YYTRANSLATE (yychar);
+      yydestruct ("Cleanup: discarding lookahead",
+                  yytoken, &yylval);
+    }
   /* Do not reclaim the symbols of the rule which action triggered
      this YYABORT or YYACCEPT.  */
   YYPOPSTACK (yylen);
@@ -1850,7 +1900,7 @@ yyreturn:
 
 
 
-/* Line 1675 of yacc.c  */
-#line 111 "pbc/parser.y"
+/* Line 2067 of yacc.c  */
+#line 112 "pbc/parser.y"
 
 
