@@ -145,6 +145,10 @@ static val_ptr v_builtin(val_ptr v, tree_ptr t) {
   return fun->run(arg);
 }
 
+static void eval_stmt(void *ptr) {
+  tree_eval(ptr);
+}
+
 static val_ptr v_def_call(val_ptr v, tree_ptr t) {
   int i;
   const char* name = ((tree_ptr) darray_at(v->def->child, 0))->id;
@@ -161,9 +165,6 @@ static val_ptr v_def_call(val_ptr v, tree_ptr t) {
   }
   // Evaluate function body.
   darray_ptr a = ((tree_ptr) darray_at(v->def->child, 2))->child;
-  void eval_stmt(void *ptr) {
-    tree_eval(ptr);
-  }
   darray_forall(a, eval_stmt);
   return NULL;
 }
@@ -175,7 +176,12 @@ static val_ptr v_field_cast(val_ptr v, tree_ptr t) {
   if (e->field == M) {
     if (v->field == M) return x;
     element_ptr e2 = element_new(v->field);
-    element_set_multiz(e2, e->data);
+    if (element_is0(e)) // if 'set0' is not 'set1' in base field of GT, but we hope 'GT(0)' calls 'set1', we may directly call 'element_set0' here
+      element_set0(e2);
+    else if (element_is1(e)) // reason is same as above
+      element_set1(e2);
+    else
+      element_set_multiz(e2, e->data);
     x->elem = e2;
     return x;
   }
@@ -287,28 +293,49 @@ static val_ptr fun_cmp(val_ptr v[], int (*fun)(int)) {
   return v[0];
 }
 
+static int is0(int i) {
+  return i == 0;
+}
+
+static int isnot0(int i) {
+  return i != 0;
+}
+
+static int isle(int i) {
+  return i <= 0;
+}
+
+static int isge(int i) {
+  return i >= 0;
+}
+
+static int islt(int i) {
+  return i < 0;
+}
+
+static int isgt(int i) {
+  return i > 0;
+}
+
 static val_ptr run_eq(val_ptr v[]) {
-  int is0(int i) { return i == 0; }
   return fun_cmp(v, is0);
 }
+
 static val_ptr run_ne(val_ptr v[]) {
-  int isnot0(int i) { return i != 0; }
   return fun_cmp(v, isnot0);
 }
+
 static val_ptr run_le(val_ptr v[]) {
-  int isle(int i) { return i <= 0; }
   return fun_cmp(v, isle);
 }
+
 static val_ptr run_ge(val_ptr v[]) {
-  int isge(int i) { return i >= 0; }
   return fun_cmp(v, isge);
 }
 static val_ptr run_lt(val_ptr v[]) {
-  int islt(int i) { return i < 0; }
   return fun_cmp(v, islt);
 }
 static val_ptr run_gt(val_ptr v[]) {
-  int isgt(int i) { return i > 0; }
   return fun_cmp(v, isgt);
 }
 
@@ -597,7 +624,8 @@ static val_ptr run_order(val_ptr v[]) {
   element_set_mpz(e, f->order);
   return val_new_element(e);
 }
-static fun_t fun_order = {{ "ord", run_order, 1, sig_field }};
+static fun_t fun_ord = {{ "ord", run_order, 1, sig_field }};
+static fun_t fun_order = {{ "order", run_order, 1, sig_field }};
 
 static val_ptr run_random(val_ptr v[]) {
   element_ptr e = pbc_malloc(sizeof(*e));
@@ -605,7 +633,8 @@ static val_ptr run_random(val_ptr v[]) {
   element_random(e);
   return val_new_element(e);
 }
-static fun_t fun_random = {{ "rnd", run_random, 1, sig_field }};
+static fun_t fun_rnd = {{ "rnd", run_random, 1, sig_field }};
+static fun_t fun_random = {{ "random", run_random, 1, sig_field }};
 
 static val_ptr run_sqrt(val_ptr v[]) {
   // TODO: Check v[0] is square.
@@ -764,6 +793,13 @@ static char *gparam =
 "coeff4 405367866213598664862417230702935310328613596\n"
 "nqr 22204504160560785687198080413579021865783099\n";
 
+static char *iparam =
+"type i\n"
+"m 97\n"
+"t 12\n"
+"n 2726865189058261010774960798134976187171462721\n"
+"n2 7\n";
+
 static val_ptr run_init_pairing_a(val_ptr v[]) {
   UNUSED_VAR(v);
   init_pairing(aparam);
@@ -809,8 +845,17 @@ static fun_t fun_init_pairing_g = {{
     "init_pairing_g", run_init_pairing_g, 0, NULL
     }};
 
-static void builtin(fun_ptr fun, const char *s) {
-  symtab_put(reserved, val_new_fun(fun), s);
+static val_ptr run_init_pairing_i(val_ptr v[]) {
+  UNUSED_VAR(v);
+  init_pairing(iparam);
+  return NULL;
+}
+static fun_t fun_init_pairing_i = {{
+    "init_pairing_i", run_init_pairing_i, 0, NULL
+    }};
+
+static void builtin(fun_ptr fun) {
+  symtab_put(reserved, val_new_fun(fun), fun->name);
 }
 
 int end_of_input;
@@ -860,26 +905,27 @@ int main(int argc, char **argv) {
   field_init_multiz(M);
   symtab_init(tab);
 
-  builtin(fun_random, "rnd");
-  builtin(fun_random, "random");
-  builtin(fun_order, "ord");
-  builtin(fun_order, "order");
-  builtin(fun_nextprime, "nextprime");
-  builtin(fun_sqrt, "sqrt");
-  builtin(fun_inv, "inv");
-  builtin(fun_type, "type");
-  builtin(fun_pairing, "pairing");
-  builtin(fun_zmod, "zmod");
-  builtin(fun_poly, "poly");
-  builtin(fun_polymod, "polymod");
-  builtin(fun_extend, "extend");
-  builtin(fun_exit, "exit");
-  builtin(fun_CHECK, "CHECK");
-  builtin(fun_init_pairing_a, "init_pairing_a");
-  builtin(fun_init_pairing_d, "init_pairing_d");
-  builtin(fun_init_pairing_e, "init_pairing_e");
-  builtin(fun_init_pairing_f, "init_pairing_f");
-  builtin(fun_init_pairing_g, "init_pairing_g");
+  builtin(fun_rnd);
+  builtin(fun_random);
+  builtin(fun_ord);
+  builtin(fun_order);
+  builtin(fun_nextprime);
+  builtin(fun_sqrt);
+  builtin(fun_inv);
+  builtin(fun_type);
+  builtin(fun_pairing);
+  builtin(fun_zmod);
+  builtin(fun_poly);
+  builtin(fun_polymod);
+  builtin(fun_extend);
+  builtin(fun_exit);
+  builtin(fun_CHECK);
+  builtin(fun_init_pairing_a);
+  builtin(fun_init_pairing_d);
+  builtin(fun_init_pairing_e);
+  builtin(fun_init_pairing_f);
+  builtin(fun_init_pairing_g);
+  builtin(fun_init_pairing_i);
   run_init_pairing_a(NULL);
   symtab_put(reserved, val_new_field(M), "M");
   symtab_put(reserved, val_new_field(Z), "Z");
